#include "project.h"
#include "stdio.h"
#include "stdlib.h"
#include "tty_utils.h"
#include "i2c_utils.h"

#define DEFAULT_I2C_SLAVE_ADDRESS 0x08

/*******************************************************************************
* Global variables
*******************************************************************************/
/* Structure for master transfer configuration */
static const cy_stc_scb_i2c_master_xfer_config_t masterTransferCfg =
{
    .slaveAddress = DEFAULT_I2C_SLAVE_ADDRESS,
    .buffer       = NULL,
    .bufferSize   = 0U,
    .xferPending  = false
};

static uint32_t i2c_timeout = 500 ; /* ms */
static int      i2c_keep_on  = 1 ;
static uint16_t i2c_loop     = 1 ;
static uint16_t i2c_interval = 100 ; /* 100 ms */
static int      i2c_init_flag = 0 ;
static uint8_t  i2c_slave_address = DEFAULT_I2C_SLAVE_ADDRESS ;
static int      i2c_delay = 30 ; /* default 30ms */

f_list_type i2c_func_list[] = {
    { "help",      do_i2c_help,       "show help message" },
    { "loop",      do_i2c_loop,       "set loop counts"},
    { "interval",  do_i2c_interval,   "set interval time (ms)"},    
    { "scan",      do_i2c_scan,       "scan I2C bus"      },
    { "slave",     do_i2c_slave,      "set slave address" },
    { "read",      do_i2c_read_byte,  "read 1 byte from the slave" },
    { "write",     do_i2c_write_byte, "write 1 byte to the slave" },
    { "writereg",  do_i2c_write_reg,  "write 1 byte to reg" },
    { "readreg",   do_i2c_read_reg,   "read 1 byte from reg" },
    { "readregs",  do_i2c_read_regs,  "read n bytes starting from the reg" },
    { "delay",     do_i2c_delay,      "set delay between address and command" },
    { "status",    do_i2c_status,     "print config status" },
    { "return",    do_i2c_return,     "return to the previous mode" },
    { 0,          0,             0} /* list terminator */
} ;

void do_i2c(void)
{
    func_ptr func ;
    i2c_keep_on = 1 ;
    
    if (i2c_init_flag == 0) {
        if (CY_SCB_I2C_SUCCESS == init_i2c_master()) {
            i2c_init_flag = 1 ;
        } else {
            print("I2C initialization failed!\n\r") ;
            return ;
        }
    }
    
    do_i2c_help() ;
    
    prompt("i2c> ") ;
    
    while(i2c_keep_on) {
        if (get_line()) {
            func = get_func(i2c_func_list, str) ;
            if (func) {
                func() ;
            } else {
                print("Unknown Command: ") ;
                print(str) ;
                print("\n\r") ;
                do_i2c_help() ;
            }
            if (i2c_keep_on) {
                prompt("i2c> ") ;
            }
        }
    }
}


void do_i2c_help(void) 
{
    show_help(i2c_func_list) ;    
}

void do_i2c_return(void) 
{
    i2c_keep_on = 0 ;
}

void do_i2c_loop(void) 
{
    sscanf(str, "%*s %hd", &i2c_loop) ;
}

void do_i2c_interval(void) 
{
    sscanf(str, "%*s %hd", &i2c_interval) ;
}

void do_i2c_delay(void)
{
    sscanf(str, "%*s %d", &i2c_delay) ;    
}

void do_i2c_status(void) 
{
    snprintf(str, STR_BUF_LEN, "I2C Config: slave address: 0x%02X (%d) ",
        i2c_slave_address, i2c_slave_address) ;
    print(str) ;
    snprintf(str, STR_BUF_LEN, "delay: %d ms\n\r", i2c_delay) ;
    print(str) ;
}

void do_i2c_scan(void) 
{
    uint8_t address ;
    print("Scanning I2C Addresses... \n\r") ;
    for (address = 1 ; address < 127 ; address++ ) {
        if (i2c_test_address(address)) {
            snprintf(str, STR_BUF_LEN, "0x%02X : device present\r\n", address) ;
            print(str) ;
        }
    }
    print("Scan completed\n\r") ;
}

void do_i2c_slave(void) 
{
    int slave_address ;
    sscanf(str, "%*s %x", &slave_address) ;
    i2c_set_slave_address(slave_address) ;
}

void do_i2c_read_byte(void) 
{
    int i ;
    cy_en_scb_i2c_status_t  errorStatus;
    uint8_t value ;
    for (i = 0 ; i < i2c_loop ; i++ ) {
        errorStatus = i2c_read_byte(&value) ;   
        snprintf(str, STR_BUF_LEN, "0x%02X\n\r", value) ;
        CyDelay(i2c_interval) ;
    }
    print(str) ;
}

void do_i2c_write_byte(void) 
{
    cy_en_scb_i2c_status_t  errorStatus;
    int value ;
    sscanf(str, "%*s %x", &value) ;
    errorStatus = i2c_write_byte(value) ;   
    if (errorStatus == CY_SCB_I2C_SUCCESS) {
        print("OK\n\r") ;
    } else {
        snprintf(str, STR_BUF_LEN, "Error: 0x%X\n\r", errorStatus) ;
        print(str) ;
    }
}

void do_i2c_read_reg(void) 
{
    int i ;
    cy_en_scb_i2c_status_t  errorStatus;
    int reg_address ;
    uint8_t value ;
    
    sscanf(str, "%*s %x", &reg_address) ;
    
    for (i = 0 ; i < i2c_loop ; i++ ) {
        errorStatus = i2c_read_reg(reg_address, &value) ;
        if (errorStatus == CY_SCB_I2C_SUCCESS) {
            snprintf(str, STR_BUF_LEN, "0x%02X (%d) \n\r",
                value, value) ;
            print(str) ;
        } else {
            snprintf(str, STR_BUF_LEN, "Error: 0x%X\n\r", errorStatus) ;
            print(str) ;
            break ;
        } 
        CyDelay(i2c_interval) ;
    }
}

void do_i2c_read_regs(void) 
{
    int loop ;
    int reg_address ;
    int num, i, tmp ;
    uint8_t *data ;
    sscanf(str, "%*s %x %d", &reg_address, &num) ;
    data = (uint8_t *)malloc(sizeof(uint8_t) * num) ;
    if (data == 0) {
        print("Not enough memory\r\n") ;
        return ;
    }
    for (loop = 0 ; loop < i2c_loop ; loop++ ) {
        i2c_read_regs(reg_address, data, num) ;
        for (i = 0 ; i < num ; i++) {
            snprintf(str, STR_BUF_LEN, "%02X ", data[i] & 0xFF) ;
            print(str) ;
        }
        print("\n\r") ;
        CyDelay(i2c_delay) ;
    }
    free( data ) ;
}

void do_i2c_write_reg(void) 
{
    cy_en_scb_i2c_status_t  errorStatus;
    int reg_address ;
    int value ;
    sscanf(str, "%*s %x %x", &reg_address, &value) ;
    errorStatus = i2c_write_reg(reg_address, value) ;
    if (errorStatus == CY_SCB_I2C_SUCCESS) {
        print("OK\n\r") ;
    } else {
        snprintf(str, STR_BUF_LEN, "Error: 0x%X\n\r", errorStatus) ;
        print(str) ;
    }
}

void i2c_set_slave_address(uint8_t address)
{
    i2c_slave_address = address ;
}

int  i2c_test_address(uint8_t address) 
{
    int result = 0 ;
    uint8_t data[2] = { 0, 0 } ;
    
    cy_en_scb_i2c_status_t  errorStatus;  
    errorStatus = Cy_SCB_I2C_MasterSendStart(I2C_HW, address, CY_SCB_I2C_WRITE_XFER, i2c_timeout, &I2C_context);
#if 0
    if (errorStatus == CY_SCB_I2C_SUCCESS) { 
        errorStatus = Cy_SCB_I2C_MasterWriteByte(I2C_HW, data[0], i2c_timeout, &I2C_context) ;
    }
    if (errorStatus == CY_SCB_I2C_SUCCESS) { 
        errorStatus = Cy_SCB_I2C_MasterSendReStart(I2C_HW, address, CY_SCB_I2C_READ_XFER, i2c_timeout, &I2C_context);
    }
    CyDelay(30) ;
    if (errorStatus == CY_SCB_I2C_SUCCESS) { 
        errorStatus = Cy_SCB_I2C_MasterReadByte(I2C_HW, CY_SCB_I2C_ACK, &data[0], i2c_timeout, &I2C_context) ;
    }  
    if (errorStatus == CY_SCB_I2C_SUCCESS) {     
        errorStatus = Cy_SCB_I2C_MasterReadByte(I2C_HW, CY_SCB_I2C_NAK, &data[1], i2c_timeout, &I2C_context) ;
    }
#endif
    Cy_SCB_I2C_MasterSendStop(I2C_HW, i2c_timeout, &I2C_context) ;      
    if (errorStatus == CY_SCB_I2C_SUCCESS) {
        result = 1 ;
    }
    return( result ) ;
}

int init_i2c_master(void) 
{
    int result = 0 ;
    volatile cy_stc_sysint_t *my_I2C_SCB_IRQ_cfg ;
    cy_en_scb_i2c_status_t initStatus ;
    cy_en_sysint_status_t sysStatus;
    uint32_t dataRateSet ;
    
    my_I2C_SCB_IRQ_cfg = &I2C_SCB_IRQ_cfg ;
    
    /* Initilaize and enable the I2C in master mode. */  
    initStatus = Cy_SCB_I2C_Init(I2C_HW, &I2C_config, &I2C_context);  
    if (initStatus != CY_SCB_I2C_SUCCESS) {
        return(initStatus) ;
    }
    dataRateSet = Cy_SCB_I2C_SetDataRate(I2C_HW, I2C_DATA_RATE_HZ, I2C_CLK_FREQ_HZ);   
    if (dataRateSet > I2C_DATA_RATE_HZ) {
        result = 1 ;
        return( result ) ;
    }
    sysStatus = Cy_SysInt_Init(&I2C_SCB_IRQ_cfg, &I2C_Interrupt);
    if (sysStatus != CY_SYSINT_SUCCESS) {
        return(sysStatus) ;
    }
    /* Enable interrupt in NVIC. */
    NVIC_EnableIRQ((IRQn_Type) I2C_SCB_IRQ_cfg.intrSrc);
    
    /* Enable I2C master hardware. */
    Cy_SCB_I2C_Enable(I2C_HW);    
    
    return(result) ;
}

cy_en_scb_i2c_status_t i2c_read_byte(uint8_t *value)
{
    cy_en_scb_i2c_status_t  errorStatus;
    errorStatus = Cy_SCB_I2C_MasterSendStart(I2C_HW, i2c_slave_address, CY_SCB_I2C_READ_XFER, i2c_timeout, &I2C_context);
       
    if (errorStatus == CY_SCB_I2C_SUCCESS) {
        CyDelay(i2c_delay) ;
        errorStatus = Cy_SCB_I2C_MasterReadByte(I2C_HW, CY_SCB_I2C_ACK, value, i2c_timeout, &I2C_context) ; 
    } else {
        print("I2C Send Start Failed!\n\r") ;
    }   
    if (errorStatus != CY_SCB_I2C_SUCCESS) {
        print("I2C Read Byte Failed\n\r") ;
    }
    errorStatus = Cy_SCB_I2C_MasterSendStop(I2C_HW, i2c_timeout, &I2C_context) ;  
    return(errorStatus) ;
}

cy_en_scb_i2c_status_t i2c_write_byte(uint8_t value)
{
    cy_en_scb_i2c_status_t  errorStatus;

    errorStatus = Cy_SCB_I2C_MasterSendStart(I2C_HW, i2c_slave_address, CY_SCB_I2C_WRITE_XFER, i2c_timeout, &I2C_context);
    if (errorStatus == CY_SCB_I2C_SUCCESS) {
        errorStatus = Cy_SCB_I2C_MasterWriteByte(I2C_HW, value, i2c_timeout, &I2C_context) ;
    } else {
        print("I2C Send Start Failed!\n\r") ;
    }

    if (errorStatus != CY_SCB_I2C_SUCCESS) {
        print("I2C Write Byte Failed\n\r") ;
    }
    errorStatus = Cy_SCB_I2C_MasterSendStop(I2C_HW, i2c_timeout, &I2C_context) ;
    return(errorStatus) ;
}

cy_en_scb_i2c_status_t i2c_read_reg(uint8_t reg_address, uint8_t *value) 
{
    cy_en_scb_i2c_status_t  errorStatus;
    errorStatus = Cy_SCB_I2C_MasterSendStart(I2C_HW, i2c_slave_address, CY_SCB_I2C_WRITE_XFER, i2c_timeout, &I2C_context);
    if (errorStatus == CY_SCB_I2C_SUCCESS) { 
        errorStatus = Cy_SCB_I2C_MasterWriteByte(I2C_HW, reg_address, i2c_timeout, &I2C_context) ;       
    } else {
        print("I2C Send Start Failed!\n\r") ;
    }

    if (errorStatus == CY_SCB_I2C_SUCCESS) {     
        CyDelay(i2c_delay) ;
        errorStatus = Cy_SCB_I2C_MasterSendReStart(I2C_HW, i2c_slave_address, CY_SCB_I2C_READ_XFER, i2c_timeout, &I2C_context);  
    } else {
        print("I2C Read Reg Sending Address Failed\n\r") ;
    }

    if (errorStatus == CY_SCB_I2C_SUCCESS) {   
        CyDelay(i2c_delay) ;
        errorStatus = Cy_SCB_I2C_MasterReadByte(I2C_HW, CY_SCB_I2C_NAK, value, i2c_timeout, &I2C_context) ;    
    } else {       
        print("I2C Send ReStart Failed!\n\r") ;
        return(errorStatus) ;
    }

    if (errorStatus != CY_SCB_I2C_SUCCESS) {
        print("I2C Read Byte Failed\n\r") ;
        return(errorStatus) ;
    }    
    errorStatus = Cy_SCB_I2C_MasterSendStop(I2C_HW, i2c_timeout, &I2C_context) ;    
    return(errorStatus) ;    
}

cy_en_scb_i2c_status_t i2c_read_regs(uint8_t reg_address, uint8_t *value, int num) 
{
    int i ;
    
    cy_en_scb_i2c_status_t  errorStatus;   
    errorStatus = Cy_SCB_I2C_MasterSendStart(I2C_HW, i2c_slave_address, CY_SCB_I2C_WRITE_XFER, i2c_timeout, &I2C_context);
    if (errorStatus == CY_SCB_I2C_SUCCESS) {
        errorStatus = Cy_SCB_I2C_MasterWriteByte(I2C_HW, reg_address, i2c_timeout, &I2C_context) ;     
        CyDelay(i2c_delay) ;
    } else {
        print("I2C Send Start Failed!\n\r") ;   
    }

    if (errorStatus == CY_SCB_I2C_SUCCESS) {
        errorStatus = Cy_SCB_I2C_MasterSendReStart(I2C_HW, i2c_slave_address, CY_SCB_I2C_READ_XFER, i2c_timeout, &I2C_context);        
        CyDelay(i2c_delay) ;
        } else {
        print("I2C Read Reg Sending Address Failed\n\r") ;
    }
        
    if (errorStatus == CY_SCB_I2C_SUCCESS) {
        for (i = 0 ; i < num - 1 ; i++ ) {
            errorStatus = Cy_SCB_I2C_MasterReadByte(I2C_HW, CY_SCB_I2C_ACK, &value[i], i2c_timeout, &I2C_context) ;
            if (errorStatus != CY_SCB_I2C_SUCCESS) {
                break ;
            }
        }
        if (errorStatus == CY_SCB_I2C_SUCCESS) {
            errorStatus = Cy_SCB_I2C_MasterReadByte(I2C_HW, CY_SCB_I2C_NAK, &value[i], i2c_timeout, &I2C_context) ;  
        }
        if (errorStatus != CY_SCB_I2C_SUCCESS) {
            print("I2C Read Byte Failed\n\r") ;
                return(errorStatus) ;
        }
    } else {
        print("I2C Send ReStart Failed!\n\r") ;     
    }
    
    errorStatus = Cy_SCB_I2C_MasterSendStop(I2C_HW, i2c_timeout, &I2C_context) ;    
    return(errorStatus) ;     
}

cy_en_scb_i2c_status_t i2c_write_reg(uint8_t reg_address, uint8_t value) 
{
    cy_en_scb_i2c_status_t  errorStatus;
    errorStatus = Cy_SCB_I2C_MasterSendStart(I2C_HW, i2c_slave_address, CY_SCB_I2C_WRITE_XFER, i2c_timeout, &I2C_context);
    if (errorStatus == CY_SCB_I2C_SUCCESS) {
        errorStatus = Cy_SCB_I2C_MasterWriteByte(I2C_HW, reg_address, i2c_timeout, &I2C_context) ;  
        CyDelay(i2c_delay) ;
    } else {
        print("I2C Send Start Failed!\n\r") ;      
    }

    if (errorStatus == CY_SCB_I2C_SUCCESS) {
        errorStatus = Cy_SCB_I2C_MasterWriteByte(I2C_HW, value, i2c_timeout, &I2C_context) ;  
    } else {
        print("I2C Write Reg Sending Address Failed\n\r") ;
    }

    if (errorStatus != CY_SCB_I2C_SUCCESS) {
        print("I2C Write Reg Failed\n\r") ;
        return(errorStatus) ;
    }    
    errorStatus = Cy_SCB_I2C_MasterSendStop(I2C_HW, i2c_timeout, &I2C_context) ;    
    return(errorStatus) ;     
}
